import React, { useRef, useEffect, useState, RefObject } from 'react';
import styles from './index.module.scss';
import hightlight from 'highlight.js';
import 'highlight.js/styles/github.css';

type props = {
  language?: string,
  debouneTime?: number,
  cb?: (content: string) => any,
  className?: string,
  content?: string,
}

const TAB = 'Tab';

const useUpdateContent = (content: string) => {
  const [_content, setContent] = useState('');
  useEffect(() => {
    setContent(content)
  }, [content])
  return { _content, setContent };
}

const useHighLight = (_content: string) => {
  const ref = useRef<HTMLElement>(null);
  useEffect(() => {
    const target = ref.current;
    if (!target) { return }
    hightlight.highlightElement(target);
  }, [_content, ref])
  return ref;
}

const useInput = (_content: string, cb: (content: string) => any, debouneTime: number) => {
  const refTimer = useRef<ReturnType<typeof setTimeout>>();
  const refDidMount = useRef(false);
  const refInputMark = useRef(false);
  useEffect(() => {
    if (!refDidMount.current) {
      refDidMount.current = true;
      cb(_content);
    } else if (refInputMark.current) {
      refInputMark.current = false;
      refTimer.current && clearTimeout(refTimer.current);
      refTimer.current = setTimeout(() => cb(_content), debouneTime);
    }
  }, [_content, refInputMark])
  return { refInputMark };
}

export const HighLighter = ({ debouneTime = 666, language = 'javascript', cb = () => { },
  className = '', content = '' }: props) => {
  const { _content, setContent } = useUpdateContent(content);
  const { refInputMark } = useInput(_content, cb, debouneTime);
  const ref = useHighLight(_content);
  const refPre = useRef<HTMLPreElement>(null);
  const [height, setHeight] = useState(0);

  const keydown = (e: React.KeyboardEvent<HTMLTextAreaElement>) => {
    if (e.key === TAB) {
      e.preventDefault();
      document.execCommand("insertText", false, '\t');
    }
  }

  const change = (e: React.ChangeEvent<HTMLTextAreaElement>) => {
    setContent(e.currentTarget.value);
    refInputMark.current = true;
  }

  return <div className={`${styles.container} ${className}`}>
    <textarea
      className={styles.input}
      value={_content}
      spellCheck='false'
      onChange={change}
      onKeyDown={keydown}
      onScroll={e => setHeight(e.currentTarget.scrollTop)}
    />
    <pre ref={refPre} className={styles.output} >
      <code className={language} ref={ref} style={{ top: `${-height}px`, position: 'absolute' }}>
        {_content}
      </code>
    </pre>
  </div>
}